#include <REG52.H>
#include <string.h>
#include <intrins.h>
#include "uart.h"

static volatile unsigned char read_buffer[READBUF_SIZE];
static volatile unsigned char read_tmp_char;
static volatile unsigned char write_rbuf_index=0;
static volatile unsigned char read_rbuf_index=0;

static volatile unsigned char write_buffer[WRITEBUF_SIZE];
static volatile unsigned char write_tmp_char;
static volatile unsigned char write_wbuf_index=0;
static volatile unsigned char read_wbuf_index=0;


/****************************************************************
**函数名称： com_init()串口初始化函数
**函数功能： 在系统时钟为11.059MHZ时，设定串口波特率为9600bit/s。
**          串口接收与发送中断允许。
**          设置中断优先级为高。
**          关闭read_flag和write_flag信号量。
*****************************************************************/

void com_init(){
	
		SCON = 0x50;           //选择串口工作方式1，打开接收允许		SM0=0x0;SM1=0x1;REN=0x1;
    TMOD = 0x20;           //定时器1工作在方式2，定时器0工作在方式1
    TH1  = 0xFA;            //实现波特率4800（系统时钟12MHZ）
    PCON = 0x00;						
    TR1 = 0x1;               //启动定时器T1
    ET1 = 0x0;
    ES = 0x1;                 //允许串行口中断
		PS = 0x1;                 //设计串行口中断优先级
    EA = 0x1;                //单片机中断允许

}

/****************************************************************
**函数名称： com_read_interrupt()串口中断处理函数
**函数功能： 处理读取完毕中断
**          处理写入完毕中断
**返回值：  读取队列存储于数组read_buffer
**          写入队列存储于数组write_buffer
*****************************************************************/

void com_interrupt(void) interrupt 4 {
		if(TI){
		    TI=0;
			if(write_wbuf_index!=read_wbuf_index){
				write_tmp_char=*(write_buffer+read_wbuf_index);
				read_wbuf_index=(++read_wbuf_index)%WRITEBUF_SIZE;
				SBUF=write_tmp_char;
			}
    }

    if(RI){
        RI=0;
        read_tmp_char = SBUF;
				if((write_rbuf_index+1)%READBUF_SIZE!=read_rbuf_index){
					*(read_buffer+write_rbuf_index)=read_tmp_char;
					write_rbuf_index=((++write_rbuf_index) % READBUF_SIZE);
				}

    }

}
/****************************************************************
**函数名称: com_write_ch
**函数功能: 向写入缓冲区写入1Byte。
**
*****************************************************************/
static COM_BUFFER_STATUS com_write_ch(unsigned char ch){
	
	if((write_wbuf_index+1)%WRITEBUF_SIZE!=read_wbuf_index){
		*(write_buffer+write_wbuf_index)=ch;
		write_wbuf_index=(++write_wbuf_index)%WRITEBUF_SIZE;
		return OK;
	}else{
		return NG;
}
}

/****************************************************************
**函数名称: com_write_str()
**函数功能: 向写入缓冲区写入数据。
**
*****************************************************************/
void com_write_str(unsigned char *str,unsigned char len){
	
	unsigned char *tmp_str=str;
	while(len--!=0){
		while(com_write_ch(*tmp_str++)!=OK);
	}
	TI=1;
}

/****************************************************************
**函数名称: com_read_ch
**函数功能: 从读取缓冲区读取1Byte。
** 
*****************************************************************/
static COM_BUFFER_STATUS com_read_ch( unsigned char *ch){

	if(write_rbuf_index!=read_rbuf_index){
		*ch=*(read_buffer+read_rbuf_index);
		read_rbuf_index=(++read_rbuf_index)%READBUF_SIZE;
		return OK;
	}else{
		return NG;
	}
}

/****************************************************************
**函数名称: com_read_str()
**函数功能: 读取指定长度的字符串。
**
*****************************************************************/
void com_read_str(unsigned char *str,unsigned char len){
	
	unsigned char tmp_ch=0;
	unsigned char *tmp_str=str;
	while(len--!=0){
		while(com_read_ch(&tmp_ch)!=OK);
		*(tmp_str++)=tmp_ch;
	}
	
}

